import sys
from rag import search

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python3 cli.py <query> [--bot=<bot_name>] [--lang=<language>] [--verbose]")
        sys.exit(1)

    query = sys.argv[1]
    bot_name = "baidu"  # default bot
    lang = "chinese"  # default language
    verbose = False  # default verbosity

    for arg in sys.argv[2:]:
        if arg.startswith("--bot="):
            bot_name = arg.split("=")[1]
        elif arg.startswith("--lang="):
            lang = arg.split("=")[1]
        elif arg == "--verbose":
            verbose = True

    answer = search(query, bot_name=bot_name, lang=lang, verbose=verbose)
    print(answer)



from typing import Optional

from langchain_core.runnables import Runnable, RunnableConfig
from langchain_core.runnables.utils import Input, Output

from bot import SearchBot


class SearchBotRetriever(Runnable):
    def __init__(self, bot: SearchBot):
        self.bot = bot

    def retrieve(self, query: str):
        return self.bot.search(query)

    def invoke(self, input: Input, config: Optional[RunnableConfig] = None) -> Output:
        return self.retrieve(input)



from typing import Optional
from urllib.parse import quote_plus

import requests
from bs4 import BeautifulSoup
from concurrent.futures import ThreadPoolExecutor

headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
}


# 获取页面body内容
def retrieve_content(item, verbose):
    url = item['url']
    title = item['title']
    try:
        if verbose:
            print(f"retrieving content from {title}")
        response = requests.get(url, headers=headers, timeout=5)
        response.encoding = 'utf-8'  # 设置编码
        response.raise_for_status()  # 检查请求是否成功
        soup = BeautifulSoup(response.text, 'html.parser')
        return f"{soup.get_text(strip=True)}\nOriginURL: {url}"  # 获取文本并去除空白字符
    except Exception as e:
        if verbose:
            print(f"failed to retrieve content from {title}: {e}")
        return ""


# 抓取页面列表
def retrieve_pages(links: list, verbose=False) -> str:
    with ThreadPoolExecutor(max_workers=len(links)) as executor:
        contents = list(executor.map(lambda item: retrieve_content(item, verbose=verbose), links))
    return "\n".join(contents)


class SearchBot:
    def __init__(self, verbose=False):
        self.verbose = verbose

    def search(self, query: str) -> Optional[str]:
        links = self.retrieve_urls(query)
        if len(links) == 0:
            return None
        contents = retrieve_pages(links, verbose=self.verbose)
        if self.verbose:
            print(f"search completed, context length: {len(contents)} characters\nGenerating...")
        return contents

    def retrieve_urls(self, query: str) -> list:
        pass


class BingSearchBot(SearchBot):
    def retrieve_urls(self, query: str) -> list:
        encoded_query = quote_plus(query)
        url = f"https://www.bing.com/search?q={encoded_query}"
        response = requests.get(url, headers=headers)
        response.encoding = 'utf-8'
        response.raise_for_status()

        soup = BeautifulSoup(response.text, 'html.parser')
        body = soup.get_text(strip=True)
        if "blocked" in body:
            raise Exception("Blocked by Bing")

        results = []
        for item in soup.select('.b_algo h2 a'):
            title = item.get_text(strip=True)
            link = item['href']
            results.append({'title': title, 'url': link})
        if len(results) == 0:
            return []
        if self.verbose:
            print(f"searched {len(results)} results")
        return results


class BaiduSearchBot(SearchBot):
    def retrieve_urls(self, query: str) -> list:
        encoded_query = quote_plus(query)
        url = f"https://www.baidu.com/s?wd={encoded_query}"
        response = requests.get(url, headers=headers)
        response.encoding = 'utf-8'
        response.raise_for_status()

        soup = BeautifulSoup(response.text, 'html.parser')
        results = []
        for item in soup.select('h3.c-title a'):
            title = item.get_text(strip=True)
            link = item['href']
            results.append({'title': title, 'url': link})
        if len(results) == 0:
            return []
        if self.verbose:
            print(f"searched {len(results)} results")
        return results


def make_bot(name: str, verbose=True) -> SearchBot:
    if name == "bing":
        return BingSearchBot(verbose=verbose)
    elif name == "baidu":
        return BaiduSearchBot(verbose=verbose)
    else:
        raise ValueError(f"Unsupported search engine: {name}")



from setuptools import setup, find_packages

setup(
    name="searchbot",
    version="1.0.4",
    author="xialeistudio",
    author_email="xialeistudio@gmail.com",
    description="An AI-powered information retrieval assistant helps users answer questions based on provided context. ",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/xialeistudio/SearchBot",
    package_dir={'': 'src'},
    packages=['searchbot'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'annotated-types==0.7.0',
        'anyio==4.4.0',
        'beautifulsoup4==4.12.3',
        'bs4==0.0.2',
        'certifi==2024.7.4',
        'charset-normalizer==3.3.2',
        'distro==1.9.0',
        'exceptiongroup==1.2.2',
        'h11==0.14.0',
        'httpcore==1.0.5',
        'httpx==0.27.0',
        'idna==3.7',
        'jiter==0.5.0',
        'jsonpatch==1.33',
        'jsonpointer==3.0.0',
        'langchain-core==0.2.32',
        'langchain-ollama==0.1.1',
        'langchain-openai==0.1.21',
        'langsmith==0.1.99',
        'ollama==0.3.1',
        'openai==1.40.6',
        'orjson==3.10.7',
        'packaging==24.1',
        'pydantic==2.8.2',
        'pydantic_core==2.20.1',
        'PyYAML==6.0.2',
        'regex==2024.7.24',
        'requests==2.32.3',
        'sniffio==1.3.1',
        'soupsieve==2.6',
        'tenacity==8.5.0',
        'tiktoken==0.7.0',
        'tqdm==4.66.5',
        'typing_extensions==4.12.2',
        'urllib3==2.2.2',
    ],
    entry_points={
        'console_scripts': [
            'search-bot=cli:main',
        ],
    },
)
